@@ -1,64 +1,18 @@
 /*
- * $Header: /home/cvspublic/jakarta-commons/modeler/src/java/org/apache/commons/modeler/BaseModelMBean.java,v 1.24 2003/07/21 04:43:16 craigmcc Exp $
- * $Revision: 1.24 $
- * $Date: 2003/07/21 04:43:16 $
- *
- * ====================================================================
- *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- * [Additional notices, if required by prior licensing conditions]
- *
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 
@@ -139,7 +93,7 @@ import org.apache.commons.modeler.modules.ModelerSource;
  *
  * @author Craig R. McClanahan
  * @author Costin Manolache
- * @version $Revision: 1.24 $ $Date: 2003/07/21 04:43:16 $
+ * @version $Revision: 480402 $ $Date: 2006-11-29 04:43:23 +0000 (Wed, 29 Nov 2006) $
  */
 
 public class BaseModelMBean implements ModelMBean, MBeanRegistration {
@@ -441,7 +395,8 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
                  "Method name is null");
 
         if( log.isDebugEnabled()) log.debug("Invoke " + name);
-        Method method=(Method)invokeAttMap.get(name);
+	MethodKey mkey = new MethodKey(name, signature);
+        Method method=(Method)invokeAttMap.get(mkey);
         if( method==null ) {
             if (params == null)
                 params = new Object[0];
@@ -491,7 +446,7 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
                                               "Cannot find method " + name +
                                               " with this signature");
             }
-            invokeAttMap.put( name, method );
+            invokeAttMap.put( mkey, method );
         }
 
         // Invoke the selected method on the appropriate object
@@ -615,16 +570,10 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
         if (attrDesc == null)
             throw new AttributeNotFoundException("Cannot find attribute " + name + " descriptor");
 
-        try {
-            // XXX Is it before or after ?
-            Object oldValue=null;
-            if( getAttMap.get(name) != null )
-                oldValue=getAttribute( name );
-            sendAttributeChangeNotification(new Attribute( name, oldValue),
-                    attribute);
-        } catch( Exception ex ) {
-            log.error( "Error sending notification " + name, ex );
-        }
+        Object oldValue=null;
+        if( getAttMap.get(name) != null )
+            oldValue=getAttribute( name );
+
 
         // Extract the method from cache
         Method m=(Method)setAttMap.get( name );
@@ -690,7 +639,12 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
             throw new MBeanException
                 (e, "Exception invoking method " + name);
         }
-
+        try {
+            sendAttributeChangeNotification(new Attribute( name, oldValue),
+                    attribute);
+        } catch(Exception ex) {
+            log.error("Error sending notification " + name, ex);
+        }
         attributes.put( name, value );
         if( source != null ) {
             // this mbean is asscoiated with a source - maybe we want to persist
@@ -1339,10 +1293,18 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
         return getModelerType();
     }
 
-    public ObjectName getObjectName() {
+    public ObjectName getJmxName() {
         return oname;
     }
 
+    public String getObjectName() {
+        if (oname != null) {
+            return oname.toString();
+        } else {
+            return null;
+        }
+    }
+
     public void setRegistry(Registry registry) {
         this.registry = registry;
     }
@@ -1416,4 +1378,40 @@ public class BaseModelMBean implements ModelMBean, MBeanRegistration {
             ((MBeanRegistration)resource).postDeregister();
         }
     }
+
+    static class MethodKey {
+	private String name;
+	private String[] signature;
+
+	MethodKey(String name, String[] signature) {
+	    this.name = name;
+	    if(signature == null) {
+		signature = new String[0];
+	    }
+	    this.signature = signature;
+	}
+
+	public boolean equals(Object other) {
+	    if(!(other instanceof MethodKey)) {
+		return false;
+	    }
+	    MethodKey omk = (MethodKey)other;
+	    if(!name.equals(omk.name)) {
+		return false;
+	    }
+	    if(signature.length != omk.signature.length) {
+		return false;
+	    }
+	    for(int i=0; i < signature.length; i++) {
+		if(!signature[i].equals(omk.signature[i])) {
+		    return false;
+		}
+	    }
+	    return true;
+	}
+
+	public int hashCode() {
+	    return name.hashCode();
+	}
+    }
 }
