@@ -1,60 +1,18 @@
 /*
- * ====================================================================
- *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- * [Additional notices, if required by prior licensing conditions]
- *
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 
@@ -71,6 +29,8 @@ import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Collections;
+import java.util.Map;
 
 import javax.management.DynamicMBean;
 import javax.management.MBeanAttributeInfo;
@@ -149,16 +109,15 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      * The set of ManagedBean instances for the beans this registry
      * knows about, keyed by name.
      */
-    private HashMap descriptors = new HashMap();
+    private final HashMap descriptors = new HashMap();
 
     /** List of managed byeans, keyed by class name
      */
-    private HashMap descriptorsByClass = new HashMap();
+    private final HashMap descriptorsByClass = new HashMap();
 
     // map to avoid duplicated searching or loading descriptors 
-    private HashMap searchedPaths=new HashMap();
+    private final Map searchedPaths= Collections.synchronizedMap(new HashMap());
     
-    private Object key;
     private Object guard;
 
     // Id - small ints to use array access. No reset on stop()
@@ -203,7 +162,6 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
                 localRegistry=(Registry)perLoaderRegistries.get(key);
                 if( localRegistry == null ) {
                     localRegistry=new Registry();
-                    localRegistry.key=key;
                     localRegistry.guard=guard;
                     perLoaderRegistries.put( key, localRegistry );
                     return localRegistry;
@@ -261,9 +219,11 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      * @since 1.1
      */ 
     public void stop() {
-        descriptorsByClass = new HashMap();
-        descriptors = new HashMap();
-        searchedPaths=new HashMap();
+        synchronized(descriptors) {
+            descriptorsByClass.clear();
+            descriptors.clear();
+            searchedPaths.clear();
+        }
     }
     
     /** 
@@ -304,9 +264,8 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
     public void loadMetadata(Object source ) throws Exception {
         if( source instanceof ClassLoader ) {
             loadMetaInfDescriptors((ClassLoader)source);
-            return;
         } else {
-            registry.loadDescriptors( null, source, null );
+            loadDescriptors( null, source, null );
         }
         
     }
@@ -454,9 +413,11 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      */
     public void addManagedBean(ManagedBean bean) {
         // XXX Use group + name
-        descriptors.put(bean.getName(), bean);
-        if( bean.getType() != null ) {
-            descriptorsByClass.put( bean.getType(), bean );
+        synchronized(descriptors) {
+            descriptors.put(bean.getName(), bean);
+            if( bean.getType() != null ) {
+                descriptorsByClass.put( bean.getType(), bean );
+            }
         }
     }
 
@@ -471,10 +432,12 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      */
     public ManagedBean findManagedBean(String name) {
         // XXX Group ?? Use Group + Type
-        ManagedBean mb=((ManagedBean) descriptors.get(name));
-        if( mb==null )
-            mb=(ManagedBean)descriptorsByClass.get(name);
-        return mb;
+        synchronized(descriptors) {
+            ManagedBean mb=((ManagedBean) descriptors.get(name));
+            if( mb==null )
+                mb=(ManagedBean)descriptorsByClass.get(name);
+            return mb;
+        }
     }
     
     /**
@@ -484,7 +447,9 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      * @since 1.0
      */
     public String[] findManagedBeans() {
-        return ((String[]) descriptors.keySet().toArray(new String[0]));
+        synchronized(descriptors) {
+            return ((String[]) descriptors.keySet().toArray(new String[0]));
+        }
     }
 
 
@@ -499,13 +464,16 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
     public String[] findManagedBeans(String group) {
 
         ArrayList results = new ArrayList();
-        Iterator items = descriptors.values().iterator();
-        while (items.hasNext()) {
-            ManagedBean item = (ManagedBean) items.next();
-            if ((group == null) && (item.getGroup() == null)) {
-                results.add(item.getName());
-            } else if (group.equals(item.getGroup())) {
-                results.add(item.getName());
+        synchronized (descriptors) {
+            for (Iterator items = descriptors.values().iterator(); items.hasNext(); ) {
+                ManagedBean item = (ManagedBean) items.next();
+                if (group == null) {
+                    if (item.getGroup() == null) {
+                        results.add(item.getName());
+                    }
+                } else if (group.equals(item.getGroup())) {
+                    results.add(item.getName());
+                }
             }
         }
         String values[] = new String[results.size()];
@@ -522,8 +490,10 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      */
     public void removeManagedBean(ManagedBean bean) {
        // TODO: change this to use group/name
-        descriptors.remove(bean.getName());
-        descriptorsByClass.remove( bean.getType());
+        synchronized (descriptors) {
+            descriptors.remove(bean.getName());
+            descriptorsByClass.remove( bean.getType());
+        }
     }
 
     // -------------------- Deprecated 1.0 methods  --------------------
@@ -548,7 +518,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      * @deprecated Use the instance method
      */
     public static void setServer(MBeanServer mbeanServer) {
-        Registry.getRegistry().setServer(mbeanServer);
+        Registry.getRegistry().setMBeanServer(mbeanServer);
     }
 
     /**
@@ -589,7 +559,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
     public String getType( ObjectName oname, String attName )
     {
         String type=null;
-        MBeanInfo info=null;
+        MBeanInfo info;
         try {
             info=server.getMBeanInfo(oname);
         } catch (Exception e) {
@@ -616,7 +586,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
     public MBeanOperationInfo getMethodInfo( ObjectName oname, String opName )
     {
         String type=null;
-        MBeanInfo info=null;
+        MBeanInfo info;
         try {
             info=server.getMBeanInfo(oname);
         } catch (Exception e) {
@@ -685,7 +655,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
         }
         
         // first look for existing descriptor
-        ManagedBean managed = registry.findManagedBean(type);
+        ManagedBean managed = findManagedBean(type);
 
         // Search for a descriptor in the same package
         if( managed==null ) {
@@ -756,9 +726,12 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
         } else if( "java.lang.Integer".equals( type ) ||
                 "int".equals( type )) {
             objValue=new Integer( value );
+        } else if("java.lang.Long".equals( type ) ||
+                  "long".equals( type )) {
+            objValue = new Long( value );
         } else if( "java.lang.Boolean".equals( type ) ||
                 "boolean".equals( type )) {
-            objValue=new Boolean( value );
+            objValue=Boolean.valueOf( value );
         }
         return objValue;
     }
@@ -856,7 +829,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
                 type=bean.getClass().getName();
             }
 
-            ManagedBean managed = registry.findManagedBean(bean.getClass(), type);
+            ManagedBean managed = findManagedBean(bean.getClass(), type);
 
             // The real mbean is created and registered
             ModelMBean mbean = managed.createMBean(bean);
@@ -880,7 +853,7 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
      *
      * @param packageName
      */
-    public void loadDescriptors( String packageName, ClassLoader classLoader  ) {
+    public synchronized void loadDescriptors( String packageName, ClassLoader classLoader  ) {
         String res=packageName.replace( '.', '/');
 
         if( log.isTraceEnabled() ) {
@@ -909,12 +882,9 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
                 loadDescriptors("MbeansDescriptorsDOMSource", dURL, null);
             else
                 loadDescriptors("MbeansDescriptorsSerSource", dURL, null);
-            return;
         } catch(Exception ex ) {
             log.error("Error loading " + dURL);
         }
-
-        return;
     }
 
     /** Experimental. Will become private, some code may still use it
@@ -979,9 +949,8 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
         if( classLoader==null ) {
             classLoader=this.getClass().getClassLoader();
         }
-        
-        String className=type;
-        String pkg=className;
+
+        String pkg=type;
         while( pkg.indexOf( ".") > 0 ) {
             int lastComp=pkg.lastIndexOf( ".");
             if( lastComp <= 0 ) return;
@@ -991,7 +960,6 @@ public class Registry implements RegistryMBean, MBeanRegistration  {
             }
             loadDescriptors(pkg, classLoader);
         }
-        return;
     }
 
     private ModelerSource getModelerSource( String type )
